#ifndef DOLIBARR_CLIENT_H
#define DOLIBARR_CLIENT_H

#include <WiFi.h>
#include <HTTPClient.h>
#include <vector>
#include "DolibarrModels.h"

constexpr u_int32_t OK_RESPONSE = 200;
constexpr u_int32_t CREATED_RESPONSE = 201;
constexpr u_int32_t NO_CONTENT_RESPONSE = 204;

constexpr u_int32_t LOGIN_JSON_SIZE = 4096;
constexpr const char* LOGIN_URL = "/users/info";
constexpr u_int32_t LIST_PRODUCT_JSON_SIZE = 4096;
constexpr const char* LIST_PRODUCT_URL = "/products/";
constexpr u_int32_t GET_PRODUCT_JSON_SIZE = 4096;
constexpr const char* GET_PRODUCT_URL = "/products/{id}/";
constexpr u_int32_t GET_PRODUCT_STOCK_JSON_SIZE = 4096;
constexpr const char* GET_PRODUCT_STOCK_URL = "/products/{id}/stock/";
constexpr u_int32_t LIST_WAREHOUSE_JSON_SIZE = 4096;
constexpr const char* LIST_WAREHOUSE_URL = "/warehouses/";
constexpr u_int32_t LIST_STOCKS_MOVEMENTS_JSON_SIZE = 4096;
constexpr const char* LIST_STOCKS_MOVEMENTS_URL = "/stockmovements/?sortfield=t.rowid&sortorder=ASC&limit=100";
constexpr u_int32_t CREATE_STOCKS_MOVEMENTS_JSON_SIZE = 4096;
constexpr const char* CREATE_STOCKS_MOVEMENTS_URL = "/stockmovements/";

struct WifiConfig {
    const char* ssid;
    const char* password;
};

struct DolibarrConfig {
    const char* url;
    const char* api_key;
};

class DolibarrClient {
public:
    DolibarrClient(WifiConfig wifi_config, DolibarrConfig dolibarr_config);
    ~DolibarrClient() = default;
    int login() const;
    std::vector<models::Warehouse> *list_warehouse() const;
    std::vector<models::Product> *list_products() const;
    models::Product *get_product_by_id(const char* id_product) const;
    int create_movement(models::CreateProductStock &stock) const;
private:
    HTTPClient* httpClient{};
    struct WifiConfig wifi;
    struct DolibarrConfig dolibarr;
    int initialize_wifi() const;
    int initialize_http_client();
    HTTPClient *build_url(const String& url) const;
};

#endif //DOLIBARR_CLIENT_H
